@@ -23,7 +23,7 @@ from shapely.geometry.base import BaseGeometry
 import eodatasets3
 from eodatasets3 import documents, images, serialise, validate
 from eodatasets3.documents import find_and_read_documents
-from eodatasets3.images import FileWrite, GridSpec, MeasurementBundler
+from eodatasets3.images import FileWrite, GridSpec, MeasurementBundler, ValidDataMethod
 from eodatasets3.model import AccessoryDoc, DatasetDoc, Location, ProductDoc
 from eodatasets3.names import NamingConventions, dc_uris, namer, resolve_location
 from eodatasets3.properties import Eo3Dict, Eo3Interface
@@ -279,6 +279,18 @@ class DatasetPrepare(Eo3Interface):
                 f"Provided collection location doesn't exist: {collection_location}"
             )
 
+        #: What method to use to calculate the valid data geometry?
+        #:
+        #: Defaults to :attr:`eodatasets3.ValidDataMethod.thorough`
+        #:
+        #: You may change this property before finishing your package.
+        #:
+        #: Eg::
+        #:
+        #:    p.valid_data_method = ValidDataMethod.filled
+        #:
+        self.valid_data_method: ValidDataMethod = ValidDataMethod.thorough
+
         if not dataset:
             dataset = DatasetDoc()
         if not dataset.id:
@@ -835,7 +847,9 @@ class DatasetPrepare(Eo3Interface):
 
         crs, grid_docs, measurement_docs = self._measurements.as_geo_docs()
 
-        valid_data = self.geometry or self._measurements.consume_and_get_valid_data()
+        valid_data = self.geometry or self._measurements.consume_and_get_valid_data(
+            valid_data_method=self.valid_data_method
+        )
 
         # Avoid the messiness of different empty collection types.
         # (to have a non-null geometry we'd also need non-null grids and crses)
